"""
GistApi-Wrapper-python -- A python wrapper for GitHub's Gist API
with advanced support to easily copy-paste the url, script,clone link
without manually looking for it on GitHub
(c) 2013 Varun Malhotra. MIT License
"""

from simplegist import *

__author__ = 'Varun Malhotra'
__version__ = '1.0.1'
__license__ = 'MIT'